/*In diesem Teil des Programms wird die Vorbereitung durchgeführt, um unseren Roboter zu der gewünschten Position zu bewegen.
  Das Verfahren zur numerischen Berechnung der Roboter Position besteht aus folgenden Schritten:
  1. In jedes Gelenk wird ein Koordinatensystem gelegt, nach der Denavit-Hartenberg Konvention 
     aus diesem dann das jeweilige Armteil betrachtet wird.
  2. Bestimmung der Transformationsmatrizen für jeweils 2 benachbarte Gelenkkoordinatensysteme.
  3. Bestimmung der Transformationsmatrix für den gesamten Roboter im Bezug zum Basiskoordinatensystem.*/




//Eine Funktion zur Berechnung der Transformationsmatrix zwischen zwei Gelenken wird erstellt.
RealMatrix trafoMatrix(float a1, float d1, float b1, float p1 ) { 


  double [][]  matrixData =  {{cos(p1), -sin(p1)*cos(b1), sin(p1)*sin(b1), a1*cos(p1)}, 
    {sin(p1), cos(p1)*cos(b1), -cos(p1)*sin(b1), a1*sin(p1)}, 
    {      0, sin(b1), cos(b1), d1}, 
    {      0, 0, 0, 1}};

  RealMatrix m = MatrixUtils.createRealMatrix(matrixData);
  return m;
}

// Funktion zur Berechnung der Transformationsmatrix des gesamten Arms im Bezug zum Basisgelenk wird erstellt.
RealMatrix gelenkArm( float[] winkel ) { 

  RealMatrix ArmTeil1 = trafoMatrix(0, 4.5, HALF_PI, winkel[0]); 
  RealMatrix ArmTeil2 = trafoMatrix(26, 0, 0, winkel[1]);
  RealMatrix ArmTeil3 = trafoMatrix(24.5, 4.5, 0, winkel[2]);

  RealMatrix gesamtArmTeilMatrix = ArmTeil1.multiply(ArmTeil2.multiply(ArmTeil3));


  return gesamtArmTeilMatrix;
} 
// Funktion zur Berechnung der Endeffektor Position wird erstellt.

RealVector endEffectorPosCalc( float[] winkel ) {

  RealVector oneVector = MatrixUtils.createRealVector(new double[] { 1, 1, 1, 1}); // Fürt vom Koorordinatenursprung des Endeffektor Koordinatensystems zu dem interessanten Punkt.
  RealVector endEffectorPos = gelenkArm (winkel).operate(oneVector); //statt oneVector cameraTarget
  println("endEffectorPos"+endEffectorPos);
  return endEffectorPos;
}