/* In unserem Fall ist die Endeffektor-Position abhängig von der Änderung der einzelnen Gelenkwinkel, zur Berechnung der einzelnen Gelenke verwerden wir das
 Newton-Vefahren um die angenährten Winkelwerte zu bekommen. Dafür haben wir zuallerst eine Jacobi Funktion erstellt, die die Steigung in einem Punkt berechnet.
 Für jede berechnete Steigung wird der Funktionswert berechnet und die Differenz zwischen zwei Funktionswerten berechnet. Das Programm läuft in einem Loop bis diese Differenz
 einen bestimmtes Minimum erreicht. Dieser Wert wird dann dem übergeordneten Programmteil als Winkel zur Verfügung gestellt, um die einzelnen Gelenke des Roboters zu steuern.
 
 */

float winkelServo1;
float winkelServo2;
float winkelServo3;

public static final double MINDIFF = 0.01;
public RealMatrix jacobiBerechnung (float[] winkel) { 


  RealMatrix jacobi= MatrixUtils.createRealMatrix(3, 3)  ;

  float [] wA = winkel, wB = winkel;

  float diffStep=0.01; //Abstand zwischen zwei Funktions


  for (int i=0; i < winkel.length; i++) {
    // bereite zwei Winkel vor um die Steigung zu berechnen.
    wA[i] -= diffStep;
    RealVector eEPosA = endEffectorPosCalc(wA);

    wB[i] += diffStep;
    RealVector eEPosB = endEffectorPosCalc(wB);

    for (int j=0; j < 3; j++) {       
      jacobi.setEntry(j, i, (eEPosB.getEntry(j) - eEPosA.getEntry(j))/(2*diffStep)) ;  //Steigungsdreieck wird berechnet
    }
  }

  return jacobi;
}

RealVector gibWinkel(RealVector zielPunkt, RealVector winkelVector ) {

  RealVector zielAusBasis = getZiel(zielPunkt, new float[]{(float)winkelVector.getEntry(0), (float)winkelVector.getEntry(1), (float)winkelVector.getEntry(2)});   


  RealVector endEPosAkt = endEffectorPosCalc(new float[] {(float)winkelVector.getEntry(0), (float)winkelVector.getEntry(1), (float)winkelVector.getEntry(2)}).getSubVector(0, 3); 

  double abstand = zielAusBasis.getDistance(endEPosAkt); //

  Math.abs(abstand);
  RealMatrix jacobi = jacobiBerechnung(new float[] {(float)winkelVector.getEntry(0), (float)winkelVector.getEntry(1), (float)winkelVector.getEntry(2)});

  DecompositionSolver solver = new SingularValueDecomposition(jacobi).getSolver();
  RealVector solution = solver.solve(zielAusBasis.subtract(endEPosAkt));

  print(" WinkelSchritt ");           
  println(solution);

  RealVector neuerWinkel = winkelVector.add(solution.mapMultiply(0.01)); //Schrittweite wurde verkleinert
  println("alter WinkelVector" + winkelVector);

  println(" neuer WinkelVector " + neuerWinkel.getEntry(0), neuerWinkel.getEntry(1), neuerWinkel.getEntry(2));

  winkelServo1 = (float) neuerWinkel.getEntry(0);
  winkelServo2 = (float) neuerWinkel.getEntry(1);
  winkelServo3 = (float) neuerWinkel.getEntry(2);
  
  return neuerWinkel;
}